/*-------------------------------------------------------------------*
 * functions: q_gain_hf()                                            *
 *                                                                   *
 * Coding/Decoding of HF gain parameters with AR prediction.         *
 *                                                                   *
 * The gain vector is quantized using one-stage VQ with 4 elements.  *
 *-------------------------------------------------------------------*/

#include "typedef.h"
#include "basic_op.h"
#include "count.h"

#include "amr_plus_fx.h"
void Q_gain_hf(
  Word16 *gain,      /* input : gain of 4 subfr */
  Word16 *gain_q,    /* output: quantized gains */
  Word16 *indice       /* output: indices         */
)
{
  Word16 i, k, gain2[Q_GN_ORDER], iQ, tmp16;
  Word32 min_err, distance;


  distance = MAX_32;                  move32();
  for (k=0; k<SIZE_BK_HF; k++)
  {
    for (i=0; i<Q_GN_ORDER; i++) 
    {
      gain2[i] = sub(sub(gain[i], Dico_gain_hf[(k*Q_GN_ORDER)+i]), MEAN_GAIN_HF_FX);      move16();
    }
    min_err = 0;      move32();
    for (i=0; i<Q_GN_ORDER; i++) 
    {
      tmp16 = mult(gain2[i], 8192);           /* ensure min_err <= 31 bits */
      min_err = L_mac(min_err, tmp16, tmp16);
    }
    test();
    if (L_sub(min_err,distance) < 0)
    {
      distance = min_err;       move32();
      indice[0] = k;            move16();
    }
  }
  iQ = shl(indice[0], 2);
  for (i=0; i<Q_GN_ORDER; i++) 
  {
    gain_q[i] = add(Dico_gain_hf[iQ+i], MEAN_GAIN_HF_FX);      move16();
  }

  return;
}



/*-------------------------------------------------------------------*
 * routine:   d_gain_hf()                                            *
 *            ~~~~~~~~~                                              *
 * Decoding of gain parameters                                       *
 *-------------------------------------------------------------------*
 *  Arguments:                                                       *
 *    indice[] : indices of the two selected codebook entries        *
 *    gain[]   : quantized gains                                     *
 *-------------------------------------------------------------------*/

#define ALPHA_FX    29491   /*Q15*/

void D_gain_hf(
  Word16 indice,     /* input:  quantization indices */
  Word16 *gain_q,    /* output: quantized gains      */
  Word16 *past_q,    /* i/o   : past quantized gain (1 word) */
  Word16 bfi         /* input : Bad frame indicator  */
)
{
  Word16 i, iQ;
  Word32 Ltmp;
  
  iQ = shl(indice, 2);
  test();
  if(bfi == 0) /* Good frame */
  {
    for (i=0; i<Q_GN_ORDER; i++)
    {
      gain_q[i] = add(Dico_gain_hf[iQ+i], MEAN_GAIN_HF_FX);  move16();
    }
  }
  else /* bad frame */
  {
    /* use the past gains slightly shifted towards the means */

    *past_q = sub(mult_r(ALPHA_FX, add(*past_q ,5120)), 5120);  move16();

    for (i=0; i<Q_GN_ORDER; i++)
    {
      gain_q[i] = add( *past_q ,  MEAN_GAIN_HF_FX);   move16();
    }
  }

  Ltmp = 0;   move32();

  for (i=0; i<Q_GN_ORDER; i++) 
  {
    Ltmp = L_add(Ltmp, gain_q[i]);
  }

  *past_q = extract_l(L_shr(L_sub(Ltmp, 2875),2));

  return;
}

